import asyncio
from typing import Union

import discord
from cpl_core.configuration.configuration_abc import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_query.extension import List
from discord.ext.commands import Context

from gismo_core.abc.bot_service_abc import BotServiceABC
from gismo_core.abc.message_service_abc import MessageServiceABC
from gismo_core.configuration.server_settings import ServerSettings


class MessageService(MessageServiceABC):

    def __init__(self, config: ConfigurationABC, logger: LoggerABC, bot: BotServiceABC):
        self._config = config
        self._logger = logger
        self._bot = bot
    
    async def delete_message(self, messages: List[discord.Message]): 
        self._logger.debug(__name__, f'Try to delete {messages.count()} messages')
        for message in messages:
            await self.delete_message(message)
        self._logger.debug(__name__, 'Deleting messages finished')
        
    async def delete_message(self, message: discord.Message):
        server_st: ServerSettings = self._config.get_configuration(f'DSERVER_{message.g.id}')
        await asyncio.sleep(server_st.message_delete_timer)
        self._logger.debug(__name__, f'Try to delete message: {message.content}')
        try:
            await message.delete()
        except Exception as e:
            self._logger.error(__name__, f'Deleting message failed', e)
        else:
            self._logger.debug(__name__, f'Deleted message {message.content}')
    
    async def send_channel_message(self, channel: discord.TextChannel, message: str):
        self._logger.debug(__name__, f'Try to send message {message} to channel {channel.id}')
        try:
            msg = await channel.send(message)
            await self.delete_message(msg)
        except Exception as e:
            self._logger.error(__name__, f'Send message to channel {channel.id} failed', e)
        else:
            self._logger.debug(__name__, f'Send message to channel {channel.id}')
    
    async def send_dm_message(self, message: str, receiver: Union[discord.User, discord.Member]):
        self._logger.debug(__name__, f'Try to send message {message} to user {receiver.id}')
        try:
            await receiver.send(message)
        except Exception as e:
            self._logger.error(__name__, f'Send message to user {receiver.id} failed', e)
        else:
            self._logger.debug(__name__, f'Send message to user {receiver.id}')
    
    async def send_ctx_msg(self, ctx: Context, message: str, file: discord.File = None): pass
